/* Copyright IBM Corp. 2010, 2012  All Rights Reserved.                    */
dojo.provide("com.ibm.skins.Standard.skin");

dojo.require("com.ibm.pb.decorations.Control");
dojo.require("com.ibm.pb.decorations.ControlMenu");

/**
 * Instances of this module are created at the root DOM node for each layout control
 * whose skin markup includes a marker that contains the resource name of this module,
 * namely com.ibm.skins.Standard.skin.  It is passed the ID of the layout control
 * and the root DOM node in the constructor.  This particular skin module decorates
 * the markup with additional markup for creating a drop-down context menu for performing
 * actions on the control (such as portlet or widget actions).  It also has some logic
 * for adding a dnd handle as well as dynamically inserting the control id for
 * active-site analytics (ASA) support.
 */
dojo.declare("com.ibm.skins.Standard.skin", com.ibm.pb.decorations.Control, {

	contextMenuOpenEvent: null,
	contextMenuCloseEvent: null,
	resizerInited: false,
	
	contextMenuMarkup: 
		 '<div class="iw-iWidget iw-Standalone" id="${iWidgetMenuId}">'
			+'<a class="iw-Definition" href="${builderRoot}/widget-catalog/ContentSetMenu.xml"></a>'
			+'<span class="iw-ItemSet" title="attributes" style="display: none;">'
				+'<a class="iw-Item" href="#contextMenuID">widgetActions_${id}</a>'
				+'<a class="iw-Item" href="#menuCSSClass">lotusActionMenu</a>'
				+'<a class="iw-Item" href="#resourceType">com.ibm.mashups.iwidget.widget.IWidgetDefinition</a>'
				+'<a class="iw-Item" href="#openEvent">WidgetActions_${id}.open</a>'
				+'<a class="iw-Item" href="#closeEvent">WidgetActions_${id}.close</a>'
			+'</span>'
		+'</div>',
	
	constructor: function(id, node) {
		var mname = "constructor", lgr = this.LOGGER;
		this.id = id;
		this.iWidgetMenuId = "widgetActionsMenu_" + id;
		this.root = node;
		this.contextMenuOpenEvent = "WidgetActions_" + id + ".open";
		this.contextMenuCloseEvent = "WidgetActions_" + id + ".close";

		this.contextMenuLink = dojo.query(".decoration-contextMenuAction", node)[0];
		if(this.contextMenuLink) {
			lgr.trace(mname, "Attaching event handler to element ${0}", [this.contextMenuLink]);
			this.connect(this.contextMenuLink, "onclick", "openContextMenu");
		}
		this.dndHandle = dojo.query(".decoration-dndHandle", node)[0];
		if(this.dndHandle) {
			lgr.trace(mname, "Adding dojo dnd handle class to element ${0}", [this.dndHandle]);
			dojo.addClass(this.dndHandle, "dojoDndHandle");
		}
		var asaId = dojo.query("*[class~=asa.portlet.id]", node)[0];
		if(asaId) {
			asaId.innerHTML = id;
		}
		var titleNode = dojo.query(".decoration-title", node)[0];
		if(titleNode) {
			if(node.firstChild) {
				//node.firstChild.setAttribute("title", titleNode.innerHTML);
			}
		}
		var ariaRegion = dojo.query(".decoration-aria-region", node)[0];
		if(ariaRegion) {
			lgr.trace(mname, "Creating ARIA region for element ${0}", [ariaRegion]);
			dojo.attr(ariaRegion, "role", "region");
			dojo.attr(ariaRegion, "aria-labelledby", "title." + id);
			dojo.attr(dojo.query(".decoration-title", node)[0], "id", "title." + id);
		}
		this.loadWidgetHeight(node);
		if (com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getPageMode() == "edit") {
			// this happens when adding widget to page in edit mode
			this.createWidgetResizeHandle(id,node);
		}
		this.changeModeHandler = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.changePageMode", this, "onPageModeChanged");
	},
	
	onPageModeChanged: function(payload) {
		if(payload == "edit" && !this.resizerInited) {
			// lazy init the resizer until edit mode
			this.createWidgetResizeHandle(this.id, this.node);
		}
	},
	
	createWidgetResizeHandle: function(id,node){
		// create the skin resize handle
		dojo.require("com.ibm.pb.layout.resize.WidgetResizer");
		var rsModel = com.ibm.mashups.enabler.runtime.skin.Factory.getRuntimeSkinModel();
		//#19026 lazy find the load until edit mode
		//var skinNode = rsModel.find(id);
		var resizerCon = dojo.query(".widget-resizer-container",node)[0];
        var widgetId = id; // we can assume that window ID and control ID are always the same, even in live preview
        if (resizerCon) {
            var resizer = dojo.query(".ibmWidgetResizer",resizerCon)[0];
            if(!resizer){
                this.resizer_dijit = new com.ibm.pb.layout.resize.WidgetResizer (
                { 
					controlId: id,
					widgetId: widgetId,
                    skinNode: null,
                    forceShowHandle: true
                });
                resizerCon.appendChild(this.resizer_dijit.domNode);
				this.resizerInited = true;
            }
        }
	},
	_findNode:function(query,baseNode){
		var nodeList = dojo.query(query,baseNode);
		if(!nodeList || nodeList.length<1) return null;
		return nodeList[0];
	},
	_getMarginHeight:function(node){
		if(!node) return 0;
		else return dojo.marginBox(node).h;
	},
	
	_getContentHeight:function(node){
		if(!node) return 0;
		else return dojo.contentBox(node).h;
	},
	
    loadWidgetHeight: function(parentNode){
        var heightCheck = parentNode.style.height;
        if (heightCheck && heightCheck.length>0) {
            var titleHeight= this._getMarginHeight(this._findNode("* > .decoration-titlebar",parentNode));
            var resizerHeight= this._getMarginHeight(this._findNode("* > .widget-resizer-container",parentNode));
            var widgetNode = this._findNode("* > .lotusWidgetBody",parentNode);     
            var widgetBorderHeight = this._getMarginHeight(widgetNode) - this._getContentHeight(widgetNode);
            var newWidgetHeight=this._getContentHeight(parentNode)-widgetBorderHeight - titleHeight -resizerHeight-1;
            dojo.style(widgetNode,"height",newWidgetHeight+"px");
            com.ibm.mashups.services.ServiceManager.getService("eventService").publishEvent(com.ibm.mashups.iwidget.Constants.RESIZE_WIDGET + "." + this.id, 
                {newHeight:newWidgetHeight,"widgetId":this.id});
        } else {
			this.windowStateChanged = this.evtSvc.subscribeEvent("com.ibm.mashups.iwidget.widgetwindowstatechanged", this, "onWindowStateChanged");
		}
        
    },

	onWindowStateChanged: function(payload) {
		if (payload.oldWindowState == "minimize") {
			this.evtSvc.unsubscribeEvent(this.windowStateChanged, null);
			this.windowStateChanged = null;

			// at this point the widget has already been changed to normal so we can read out the height as we used to
			this.loadWidgetHeight(this.root);
		}
	},

	openContextMenu: function(evt) {
		var mname = "openContextMenu", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		if (!this.contextMenu) {
			this.contextMenu = new com.ibm.pb.decorations.ControlMenu(this.id, "widgetActions_" + this.id);
			var parseRoot = dojo.query(".decoration-contextMenu", this.root)[0];
			if(parseRoot) {
				var map = dojo.delegate(this, {builderRoot: ibmConfig["com.ibm.mashups.contextroot.builder"]});
				parseRoot.innerHTML = dojo.string.substitute(this.contextMenuMarkup, map);
				dojo.publish("/com/ibm/mashups/livetext/livetextchanged", [parseRoot, true]);
			}
		}
		
		// Place a slight delay on the broadcasting of the open context menu event to ensure the menu is fully initialized
		var self = this;
		setTimeout(function(){self.evtSvc.broadcastEvent(self.contextMenuOpenEvent, {resourceId: self.id, domNode: evt.target})}, 250);
		lgr.exiting(mname, arguments);
	},
	
	destroy: function() {
		this.evtSvc.broadcastEvent("com.ibm.mashups.iwidget.unloadwidgets", [this.iWidgetMenuId]);
		this.evtSvc.unsubscribeEvent(this.changeModeHandler, null);
		if (this.windowStateChanged) {
			this.evtSvc.unsubscribeEvent(this.windowStateChanged, null);
		}
		this.contextMenu.destroy();
		delete this.contextMenu;
        if (this.resizer_dijit) {
            this.resizer_dijit.destroy();
            delete this.resizer_dijit;
        }
		this.inherited(arguments);
	}
});
