dojo.provide("com.ibm.skins.Thin.skin");

dojo.require("com.ibm.pb.decorations.Control");

dojo.declare("com.ibm.skins.Thin.skin", com.ibm.pb.decorations.Control, {
	
	contextMenuOpenEvent: null,
	contextMenuCloseEvent: null,
	titleBarShowing: false,
	connections: [],
	resizerInited: false,
	
	contextMenuMarkup: 
		 '<div class="iw-iWidget iw-Standalone" id="${iWidgetMenuId}">'
			+'<a class="iw-Definition" href="${builderRoot}/widget-catalog/ContentSetMenu.xml"></a>'
			+'<span class="iw-ItemSet" title="attributes" style="display: none;">'
				+'<a class="iw-Item" href="#contextMenuID">widgetActions_${id}</a>'
				+'<a class="iw-Item" href="#menuCSSClass">lotusActionMenu</a>'
				+'<a class="iw-Item" href="#resourceType">com.ibm.mashups.iwidget.widget.IWidgetDefinition</a>'
				+'<a class="iw-Item" href="#openEvent">WidgetActions_${id}.open</a>'
				+'<a class="iw-Item" href="#closeEvent">WidgetActions_${id}.close</a>'
			+'</span>'
		+'</div>',
	
	constructor: function(id, node) {
		var mname = "constructor", lgr = this.LOGGER;
		this.id = id;
		this.iWidgetMenuId = "widgetActionsMenu_" + id;
		this.root = node;
		this.contextMenuOpenEvent = "WidgetActions_" + id + ".open";
		this.contextMenuCloseEvent = "WidgetActions_" + id + ".close";

		this.contextMenuLink = dojo.query(".decoration-contextMenuAction", node)[0];
		if(this.contextMenuLink) {
			lgr.trace(mname, "Attaching event handler to element ${0}", [this.contextMenuLink]);
			this.connect(this.contextMenuLink, "onclick", "openContextMenu");
		}
		this.dndHandle = dojo.query(".decoration-dndHandle", node)[0];
		if(this.dndHandle) {
			lgr.trace(mname, "Adding dojo dnd handle class to element ${0}", [this.dndHandle]);
			dojo.addClass(this.dndHandle, "dojoDndHandle");
		}
		var asaId = dojo.query("*[class~=asa.portlet.id]", node)[0];
		if(asaId) {
			asaId.innerHTML = id;
		}
		var ariaRegion = dojo.query(".decoration-aria-region", node)[0];
		if(ariaRegion) {
			lgr.trace(mname, "Creating ARIA region for element ${0}", [ariaRegion]);
			dojo.attr(ariaRegion, "role", "region");
			dojo.attr(ariaRegion, "aria-labelledby", "title." + id);
			dojo.attr(dojo.query(".decoration-title", node)[0], "id", "title." + id);
		}
		this.loadWidgetHeight(node);
		this.attachTitleBarHandlers(node);
		if (com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getPageMode() == "edit") {
			// this happens when adding widget to page in edit mode
			this.createWidgetResizeHandle(id,node);
		}
		this.changeModeHandler = this.evtSvc.subscribeEvent("com.ibm.mashups.builder.changePageMode", this, "onPageModeChanged");
	},
	
	onPageModeChanged: function(payload) {
		if(payload == "edit" && !this.resizerInited) {
			// lazy init the resizer until edit mode
			this.createWidgetResizeHandle(this.id, this.node);
		}
	},
	
	createWidgetResizeHandle: function(id,node){
		// create the skin resize handle
		dojo.require("com.ibm.pb.layout.resize.WidgetResizer");
		var rsModel = com.ibm.mashups.enabler.runtime.skin.Factory.getRuntimeSkinModel();
		//#19026 lazy find the load until edit mode
		//var skinNode = rsModel.find(id);
		var resizerCon = dojo.query(".widget-resizer-container",node)[0];
        var widgetId = id;// we can assume that window ID and control ID are always the same, even in live preview
        if (resizerCon) {
            var resizer = dojo.query(".ibmWidgetResizer",resizerCon)[0];
            if(!resizer){
                this.resizer_dijit = new com.ibm.pb.layout.resize.WidgetResizer (
                { 
					controlId: id,
					widgetId: widgetId,
					//#17713 memory leak for one component control node, controlNode is not used
                    //controlNode: node,
                    skinNode: null,
                    forceShowHandle: true
                });
                resizerCon.appendChild(this.resizer_dijit.domNode);
				this.resizerInited = true;
            }
        }
	},
	attachTitleBarHandlers: function(node) {
		// summary: connects js handlers to show and hide link nodes
		// node: the skin DOM node
		var controlDiv = dojo.query(".lotusWidget2", node)[0];
		var hideLink = dojo.query(".ibmHideTitleBarLink", node)[0];
		var showLink = dojo.query(".ibmShowTitleBarLink", node)[0];
		this.connections.push(dojo.connect(hideLink, "onclick", this, function(){ this.hideTitleBar(controlDiv,node); }));
		this.connections.push(dojo.connect(showLink, "onclick", this, function(){ this.showTitleBar(controlDiv,showLink,node); }));
		this.connections.push(dojo.connect(showLink, "onblur", this, function(){ this.hideShowLink(showLink); }));
		this.connections.push(dojo.connect(showLink, "onfocus", this, function(){ this.displayShowLink(showLink); }));
		this.connections.push(dojo.connect(node, "onmouseout", this, function(){ this.hideShowLink(showLink); }));
		this.connections.push(dojo.connect(node, "onmouseover", this, function(){ this.displayShowLink(showLink); }));
	},
	showTitleBar: function(controlDiv, showLink, node) {
		// summary: show the title bar and move focus onto it
		// controlDiv: the lotusWidget2 div
		// showLink: the "Show title bar" link
		// node: the root DOM node of the skin
		this.titleBarShowing = true;
		var menu = dojo.query(".decoration-contextMenuAction", node)[0];
		dojo.removeClass(controlDiv, "ibmHideTitleBar");
		dojo.addClass(showLink, "ibmHoverText");
		var height = this.getTitleBarHeight(node);
		this.addHeightToWidgetBody(height*(-1), node);
		menu.focus();
	},
	hideTitleBar: function(controlDiv, node) {
		// summary: hides the title bar
		// controlDiv: the lotusWidget2 div
		// node: the root DOM node of the skin
		this.titleBarShowing = false;
		var height = this.getTitleBarHeight(node);
		this.addHeightToWidgetBody(height, node);
		dojo.addClass(controlDiv, "ibmHideTitleBar");
	},
	displayShowLink: function(showLink) {
		// summary: shows the "Show title bar" link
		// showLink: the "Show title bar" link
		if (!this.titleBarShowing) {
			dojo.removeClass(showLink, "ibmHoverText");
		}
	},
	hideShowLink: function(showLink) {
		// summary: hides the "Show title bar" link
		// showLink: the "Show title bar" link
		if (!this.titleBarShowing) {
			dojo.addClass(showLink, "ibmHoverText");
		}
	},
	getTitleBarHeight: function(node) {
		// summary: returns the height of the title bar in pixels
		// node: the root DOM node of the skin
		var titlebar = dojo.query(".decoration-titlebar", node)[0];
		return dojo.style(titlebar,"height") +
				dojo.style(titlebar,"paddingTop") +
				dojo.style(titlebar,"paddingBottom");
	},
	addHeightToWidgetBody: function(px, node) {
		// summary: adds some pixels to widget body's inline height
		// px: number of pixels to add to the height
		// node: the root DOM node of the skin
		var widgetBody = dojo.query(".lotusWidgetBody", node)[0];
		var style = dojo.attr(widgetBody, "style");
		var styleStr = style ? style.toString().toLowerCase() : null;
		if ((style && style.height) || (styleStr && styleStr.indexOf("height") >= 0)) {
			var h = dojo.style(widgetBody, "height");
			dojo.style(widgetBody, "height", h+px+"px");
		}
	},
	_findNode:function(query,baseNode){
		var nodeList = dojo.query(query,baseNode);
		if(!nodeList || nodeList.length<1) return null;
		return nodeList[0];
	},
	_getMarginHeight:function(node){
		if(!node) return 0;
		else return dojo.marginBox(node).h;
	},
	
	_getContentHeight:function(node){
		if(!node) return 0;
		else return dojo.contentBox(node).h;
	},
	
	loadWidgetHeight: function(parentNode){
        var heightCheck = parentNode.style.height;
        if (heightCheck && heightCheck.length > 0) {
            var titleHeight = this._getMarginHeight(this._findNode("* > .decoration-titlebar", parentNode));
            var toggleTitleHeight = this._getMarginHeight(this._findNode("* > .ibmToggleTitleBar", parentNode));
            var resizerHeight = this._getMarginHeight(this._findNode("* > .widget-resizer-container", parentNode));
            var widgetNode = this._findNode("* > .lotusWidgetBody", parentNode);
            var widgetBorderHeight = this._getMarginHeight(widgetNode) - this._getContentHeight(widgetNode);
            var newWidgetHeight = this._getContentHeight(parentNode) - widgetBorderHeight - toggleTitleHeight - titleHeight - resizerHeight - 1;
            dojo.style(widgetNode, "height", newWidgetHeight + "px");
            com.ibm.mashups.services.ServiceManager.getService("eventService").publishEvent(com.ibm.mashups.iwidget.Constants.RESIZE_WIDGET + "." + this.id, 
                {newHeight:newWidgetHeight,"widgetId":this.id});
        } else {
			this.windowStateChanged = this.evtSvc.subscribeEvent("com.ibm.mashups.iwidget.widgetwindowstatechanged", this, "onWindowStateChanged");
        }
    },

	onWindowStateChanged: function(payload) {
		if (payload.oldWindowState == "minimize") {
			this.evtSvc.unsubscribeEvent(this.windowStateChanged, null);
			this.windowStateChanged = null;

			// at this point the widget has already been changed to normal so we can read out the height as we used to
			this.loadWidgetHeight(this.root);
		}
	},

	openContextMenu: function(evt) {
		// summary: opens the skin's context menu
		// evt: the js DOM event
		var mname = "openContextMenu", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		if (!this.contextMenu) {
			this.contextMenu = new com.ibm.pb.decorations.ControlMenu(this.id, "widgetActions_" + this.id);
			var parseRoot = dojo.query(".decoration-contextMenu", this.root)[0];
			if(parseRoot) {
				var map = dojo.delegate(this, {builderRoot: ibmConfig["com.ibm.mashups.contextroot.builder"]});
				parseRoot.innerHTML = dojo.string.substitute(this.contextMenuMarkup, map);
				dojo.publish("/com/ibm/mashups/livetext/livetextchanged", [parseRoot, true]);
			}
		}
		
		// Place a slight delay on the broadcasting of the open context menu event to ensure the menu is fully initialized
		var self = this;
		setTimeout(function(){self.evtSvc.broadcastEvent(self.contextMenuOpenEvent, {resourceId: self.id, domNode: evt.target})}, 250);
		lgr.exiting(mname, arguments);
	},
	destroy: function() {
		// summary: destroys the skin and cleans up
		this.evtSvc.broadcastEvent("com.ibm.mashups.iwidget.unloadwidgets", [this.iWidgetMenuId]);
		this.evtSvc.unsubscribeEvent(this.changeModeHandler, null);
		if (this.windowStateChanged) {
			this.evtSvc.unsubscribeEvent(this.windowStateChanged, null);
		}
		this.contextMenu.destroy();
		delete this.contextMenu;
        if (this.resizer_dijit) {
            this.resizer_dijit.destroy();
            delete this.resizer_dijit;
        }
		dojo.forEach(this.connections, dojo.disconnect);
		this.inherited(arguments);
	}
});
